#!/bin/bash
# Gets generic IPOPP module ready to ship
#
# If no arguments, just builds stuff
#
# If exactly two arguments, first one is destination directory
# and second one is version number

# First, find out where we are

case $0 in
         /*)  SHELLFILE=$0 ;;
        ./*)  SHELLFILE=${PWD}${0#.} ;;
        ../*) SHELLFILE=${PWD%/*}${0#..} ;;
          *)  SHELLFILE=$(type -P $0) ; if [ ${SHELLFILE:0:1} != "/" ]; then SHELLFILE=${PWD}/$SHELLFILE ; fi ;;
esac
SHELLDIR=${SHELLFILE%/*}

# So, SHELLDIR ought to be IPOPP_HOME/dsm
MODULE_HOME=${SHELLDIR}
IPOPP_HOME=${MODULE_HOME}/..

# Just to make everyone happy, cd to dsm
pushd ${MODULE_HOME}

# For generality, the name of the module is the name of the directory
# we're in (feel free to hard-code this instead)
MODULE_NAME=${PWD##*/}

if ./build.sh
then
    echo Build OK
else
    echo Build failed
    exit 1
fi

if [ -d ${IPOPP_HOME}/jsw -a -x ${IPOPP_HOME}/jsw/assemble.sh ]
then
    if ${IPOPP_HOME}/jsw/assemble.sh ${MODULE_HOME}
    then
	echo jsw assemble OK
    else
	echo jsw assemble failed
	exit 1
	fi
else
    echo The jsw module is missing or incomplete
    echo Check it out and try again
    exit 1
fi

if ./build_javadoc.sh
then
    echo javadoc OK
else
    echo javadoc build failed
    exit 1
fi

# Here's how you can go ahead and tar stuff up

if (( $# == 2 ))
then
    if [ -d $1 -a -w $1 ]
    then
	RESULTPATH=$1/${MODULE_NAME}-$2.tgz
	rm ship.sh
	cd ..
	if tar czf ${RESULTPATH} ${MODULE_NAME}
	then 
	    echo Result is at ${RESULTPATH}
	else
	    echo tar file creation failed for ${RESULTPATH}
	    exit 1
	fi
    else
	echo $1 is not a writable directory
	exit 1
    fi
else
    rm ship.sh
fi

# We're done
echo $MODULE_NAME ready to ship

popd
