package gov.nasa.gsfc.drl.rtstps.core.output.rdr;

import gov.nasa.gsfc.drl.rtstps.core.RtStpsException;

import java.text.ParseException;


//placeholder for further specialization
public class SpacecraftDiaryGranule extends Granule {
	
	
	private static long endOffset = 20000000L;
	private LPEATEDate firstDateTime;
	private LPEATEDate lastDateTime;
	private PDSDate predictedEndDateTime;
	private PDSDateTrunc beginningTimeDateTime;
	private PDSDateTrunc predictedEndingTimeDateTime;

	
	/**
	 * Constructor for a Granule instance, the arguments are associated with the granule's attributes.
	 * An instance of this class in this packet is created by some other factory method.
	 * 
	 * @param rap the corresponding RawApplicationPackets area
	 * @param orbit the orbit number of the pass
	 * @param granuleId the granuleId {@link GranuleId}
	 * @param leoaState the LEO state flag
	 * @param docName the document name of the specification controlling this granules construction
	 * @param packetTypes an array of packet types received in this granule
	 * @param packetTypeCounts the counts per type of the packets received
	 * @param referenceId the reference identifier which is a UUID {@link java.util.UUID}
	 * @param granuleNumber the granule number which corresponds to the RawApplicationPackets number in the RDR/HDF file
	 * @param dataSpaceOfRaw the HDF DataSpace handle of the RawApplicationPackets area associated with this granule
	 */
	public SpacecraftDiaryGranule(RawApplicationPackets rap,
					long orbit,
					GranuleId granuleId,
					LEOAFlag leoaState,
					String docName,
					String[] packetTypes,
					long[] packetTypeCounts,
					ReferenceId referenceId,
					int granuleNumber, 
					int dataSpaceOfRaw) {
		super(rap, 
				orbit, 
				granuleId, 
				leoaState, 
				docName, 
				packetTypes, 
				packetTypeCounts, 
				referenceId, 
				granuleNumber,  
				dataSpaceOfRaw,
				RDRName.NPP_Ephemeris_and_Attitude);
	}
	/**
	 * Read the Granule out of the HDF file and fill in the various attributes and items in it for
	 * these access methods
	 * @param groupId the HDF group handle
	 * @param granuleName  the name of the granule
	 * @throws ParseException 
	 */
	public SpacecraftDiaryGranule(int groupId, String granuleName) throws RtStpsException {
		super(groupId, granuleName);
	}
	
	@Override
	public PDSDate getBeginningObservationDateTime(RawApplicationPackets rap) {
		this.firstDateTime = new LPEATEDate(rap.getFirstTime());  // packet time
		return firstDateTime;
	}

	
	@Override
	public PDSDate getEndingObservationDateTime(RawApplicationPackets rap) {
		this.lastDateTime = new LPEATEDate(rap.getLastTime());  // this is saved for no particular reason at this time
		
		// this is the one that's returned, it's the predicated time
		// that's the first time plus the predicated offset... 
		
		this.predictedEndDateTime = firstDateTime.addMicros(endOffset);
		return predictedEndDateTime;
	}

	@Override
	public PDSDate getBeginningTimeDateTime() {
		beginningTimeDateTime = new PDSDateTrunc(firstDateTime.getOriginalPacketTime());
		
		// FIXME this should be TRUNCATED as well
		return beginningTimeDateTime;
	}
	@Override
	public PDSDate getEndingTimeDateTime() {
		this.predictedEndingTimeDateTime = beginningTimeDateTime.addMicros(endOffset);
		return predictedEndingTimeDateTime;

	}
	@Override
	long getScansPerGranule() {
		// TODO Auto-generated method stub
		return 0;
	}

}
