package gov.nasa.gsfc.drl.rtstps.core.output.rdr;

import gov.nasa.gsfc.drl.rtstps.core.RtStpsException;

import java.text.ParseException;

/**
 * This works for the SDR algorithm with the time code checks turned off -- ONE granule test only
 * @author krice
 *
 */
public class VIIRSGranule extends Granule {
 // 85 400 000
 // 85 350 000
	private final static long lpeateGranuleSize = 1778125L;  // calculated from LPEATE RDRs
	//private long scansPerGranule = 48L;
	//private long endOffset = lpeateGranuleSize * scansPerGranule;
	
	//private LPEATEDate firstDateTime;
	private LPEATEDate firstDateTime;
	private LPEATEDate lastDateTime;
	private PDSDate predictedEndDateTime;
	private PDSDateTrunc beginningTimeDateTime;
	private PDSDateTrunc predictedEndingTimeDateTime;
	
	/**
	 * Constructor for a Granule instance, the arguments are associated with the granule's attributes.
	 * An instance of this class in this packet is created by some other factory method.
	 * 
	 * @param rap the corresponding RawApplicationPackets area
	 * @param orbit the orbit number of the pass
	 * @param granuleId the granuleId {@link GranuleId}
	 * @param leoaState the LEO state flag
	 * @param docName the document name of the specification controlling this granules construction
	 * @param packetTypes an array of packet types received in this granule
	 * @param packetTypeCounts the counts per type of the packets received
	 * @param referenceId the reference identifier which is a UUID {@link java.util.UUID}
	 * @param granuleNumber the granule number which corresponds to the RawApplicationPackets number in the RDR/HDF file
	 * @param dataSpaceOfRaw the HDF DataSpace handle of the RawApplicationPackets area associated with this granule
	 */
	public VIIRSGranule(RawApplicationPackets rap,
					long orbit,
					GranuleId granuleId,
					LEOAFlag leoaState,
					String docName,
					String[] packetTypes,
					long[] packetTypeCounts,
					ReferenceId referenceId,
					int granuleNumber, 
					int dataSpaceOfRaw) {
		super(rap, 
				orbit, 
				granuleId, 
				leoaState, 
				docName, 
				packetTypes, 
				packetTypeCounts, 
				referenceId, 
				granuleNumber,  
				dataSpaceOfRaw,
				RDRName.VIIRS_Science);
		
		
	
		
	}
	
	long getEndOffSet() {
		return lpeateGranuleSize * getScansPerGranule();
	}
	
	long getScansPerGranule() {
		
		return 48;  // should be 48 but its the scan count for VIIRSRawAppPacket3 right now
	}

	/**
	 * Read the Granule out of the HDF file and fill in the various attributes and items in it for
	 * these access methods
	 * @param groupId the HDF group handle
	 * @param granuleName  the name of the granule
	 * @throws ParseException 
	 */
	public VIIRSGranule(int groupId, String granuleName) throws RtStpsException {
		super(groupId, granuleName);
	}
	@Override
	public PDSDate getBeginningObservationDateTime(RawApplicationPackets rap) {
		this.firstDateTime = new LPEATEDate(rap.getFirstTime());  // packet time
		
		
		// ignore this, from an earlier test
		//long t_microseconds = 1422168873750000L;
		//long MicrosPerDay = 86400000L * 1000L;
		//long c_Day = t_microseconds / MicrosPerDay;
		
		//t_microseconds = t_microseconds - (c_Day * MicrosPerDay);
		
		//long c_Millis = (t_microseconds / 1000L);
		
		//long c_Micros = t_microseconds - (c_Millis * 1000L);

		
		//long rawDay = c_Day;
		//long rawMillis = c_Millis;
		//long rawMicros =  c_Micros;
		

		//long rawPacketTime =   ((rawDay << 48)    & 0xffff000000000000L);
		//rawPacketTime |= ((rawMillis << 16)  & 0x0000ffffffff0000L);
		//rawPacketTime |=  (rawMicros         & 0x000000000000ffffL);
		
		
		//this.firstDateTime = new PDSDateTrunc(rawPacketTime);  // VIIRS test
		return firstDateTime;
	}

	
	@Override
	public PDSDate getEndingObservationDateTime(RawApplicationPackets rap) {
		this.lastDateTime = new LPEATEDate(rap.getLastTime());  // this is saved for no particular reason at this time
		
		// this is the one that's returned, it's the predicated time
		// that's the first time plus the predicated offset... 
		
		this.predictedEndDateTime = firstDateTime.addMicros(getEndOffSet() + 50000L);
		
		// viirs test
		//PDSDateTrunc predictedEndDateTime =  new PDSDateTrunc(lastDateTime.getOriginalPacketTime()); // VIIRS partial granule test
		//this.predictedEndDateTime = predictedEndDateTime.addMicros(1350000L);
		//return this.predictedEndDateTime;
		return this.lastDateTime;
	}
	@Override
	public PDSDate getBeginningTimeDateTime() {
		beginningTimeDateTime = new PDSDateTrunc(firstDateTime.getOriginalPacketTime());
		
		// FIXME this should be TRUNCATED as well
		return beginningTimeDateTime;
	}
	@Override
	public PDSDate getEndingTimeDateTime() {
		
		System.out.println("Endoffset == " + getEndOffSet());
		
		this.predictedEndingTimeDateTime = beginningTimeDateTime.addMicros( getEndOffSet() + 50000L);
		
		// viirs test
		//predictedEndingTimeDateTime = new PDSDateTrunc(lastDateTime.getOriginalPacketTime());
		
		//predictedEndingTimeDateTime = predictedEndingTimeDateTime.addMicros(1350000L);
		return predictedEndingTimeDateTime;

	}


}
